@@ -192,13 +192,10 @@ impl AteccDevice {
 
             start_pos += shift;
             let remaining_bytes = data.len() - start_pos;
-            match 0 == remaining_bytes {
-                true => shift = 0,
-                false => {
-                    if remaining_bytes < ATCA_AES_DATA_SIZE {
-                        shift = remaining_bytes
-                    }
-                }
+            if 0 == remaining_bytes {
+                shift = 0
+            } else if remaining_bytes < ATCA_AES_DATA_SIZE {
+                shift = remaining_bytes
             }
         }
 
@@ -297,13 +294,10 @@ impl AteccDevice {
 
             start_pos += shift;
             let remaining_bytes = data.len() - start_pos;
-            match 0 == remaining_bytes {
-                true => shift = 0,
-                false => {
-                    if remaining_bytes < ATCA_AES_DATA_SIZE {
-                        shift = remaining_bytes
-                    }
-                }
+            if 0 == remaining_bytes {
+                shift = 0
+            } else if remaining_bytes < ATCA_AES_DATA_SIZE {
+                shift = remaining_bytes
             }
         }
 
