@@ -33,18 +33,15 @@ impl AteccDevice {
                 let block = &data[start_pos..(start_pos + shift)];
                 let mut encr_block: [u8; ATCA_AES_DATA_SIZE] = [0; ATCA_AES_DATA_SIZE];
 
-                ctx = self.aes_gcm_encrypt_update(ctx, &block, &mut encr_block)?;
+                ctx = self.aes_gcm_encrypt_update(ctx, block, &mut encr_block)?;
                 data[start_pos..(shift + start_pos)].clone_from_slice(&encr_block[..shift]);
 
                 start_pos += shift;
                 let remaining_bytes = data.len() - start_pos;
-                match 0 == remaining_bytes {
-                    true => shift = 0,
-                    false => {
-                        if remaining_bytes < ATCA_AES_DATA_SIZE {
-                            shift = remaining_bytes
-                        }
-                    }
+                if 0 == remaining_bytes {
+                    shift = 0
+                } else if remaining_bytes < ATCA_AES_DATA_SIZE {
+                    shift = remaining_bytes
                 }
             }
         }
@@ -78,18 +75,15 @@ impl AteccDevice {
                 let block = &data[start_pos..(start_pos + shift)];
                 let mut encr_block: [u8; ATCA_AES_DATA_SIZE] = [0; ATCA_AES_DATA_SIZE];
 
-                ctx = self.aes_gcm_decrypt_update(ctx, &block, &mut encr_block)?;
+                ctx = self.aes_gcm_decrypt_update(ctx, block, &mut encr_block)?;
                 data[start_pos..(shift + start_pos)].clone_from_slice(&encr_block[..shift]);
 
                 start_pos += shift;
                 let remaining_bytes = data.len() - start_pos;
-                match 0 == remaining_bytes {
-                    true => shift = 0,
-                    false => {
-                        if remaining_bytes < ATCA_AES_DATA_SIZE {
-                            shift = remaining_bytes
-                        }
-                    }
+                if 0 == remaining_bytes {
+                    shift = 0
+                } else if remaining_bytes < ATCA_AES_DATA_SIZE {
+                    shift = remaining_bytes
                 }
             }
         }
@@ -150,16 +144,13 @@ impl AteccDevice {
             let mut shift: usize = min(data_to_sign.len(), ATCA_AES_DATA_SIZE);
             while shift > 0 {
                 let block = &data_to_sign[start_pos..(start_pos + shift)];
-                ctx = self.aes_gcm_aad_update(ctx, &block)?;
+                ctx = self.aes_gcm_aad_update(ctx, block)?;
                 start_pos += shift;
                 let remaining_bytes = data_to_sign.len() - start_pos;
-                match 0 == remaining_bytes {
-                    true => shift = 0,
-                    false => {
-                        if remaining_bytes < ATCA_AES_DATA_SIZE {
-                            shift = remaining_bytes
-                        }
-                    }
+                if 0 == remaining_bytes {
+                    shift = 0
+                } else if remaining_bytes < ATCA_AES_DATA_SIZE {
+                    shift = remaining_bytes
                 }
             }
         }
